package handlers

import (
	"context"
	"time"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// HandleUserActivity deals with user activity from the renderlogs table.
func HandleUserActivity(
	ctx context.Context,
	db *sqlx.DB,
	user *models.Userinfo,
	log *zerolog.Logger,
) error {
	if user.IsAdmin { // admins may have an activity but no renderstats
		user.Activity = &models.Activity{}

		return nil
	}

	var (
		username                = user.Username
		totalRenderCount        = 0
		currentMonthRenderCount = 0
		lastMonthRenderCount    = 0
		month                   = time.Now().Month()
		lastMonth               = time.Now().AddDate(0, -1, 0).Month()
	)

	totalQuery := squirrel.Select("COUNT(*)").
		From(models.RenderLogTable).
		Where(squirrel.Eq{models.RenderLogUsernameColumn: username})
	if err := database.GetContext(
		ctx, db, &totalRenderCount, totalQuery, log); err != nil {
		return err
	}

	currentMonthQuery := squirrel.Select("COUNT(*)").
		From(models.RenderLogTable).
		Where(squirrel.Eq{models.RenderLogUsernameColumn: username}).
		Where(squirrel.Expr("EXTRACT(MONTH FROM rendered_at) = ?", month))
	if err := database.GetContext(
		ctx, db, &currentMonthRenderCount, currentMonthQuery, log); err != nil {
		return err
	}

	lastMonthQuery := squirrel.Select("COUNT(*)").
		From(models.RenderLogTable).
		Where(squirrel.Eq{models.RenderLogUsernameColumn: username}).
		Where(squirrel.Expr("EXTRACT(MONTH FROM rendered_at) = ?", lastMonth))
	if err := database.GetContext(
		ctx, db, &lastMonthRenderCount, lastMonthQuery, log); err != nil {
		return err
	}

	activity := models.Activity{
		RenderStats: &models.RenderStats{
			Total:        int64(totalRenderCount),
			CurrentMonth: int64(currentMonthRenderCount),
			LastMonth:    int64(lastMonthRenderCount),
		},
	}
	user.Activity = &activity

	return nil
}

// NewUserGetHandler returns a UserGetHandler.
func NewUserGetHandler(db *sqlx.DB) *UserGetHandler {
	return &UserGetHandler{
		db: db,
	}
}

// UserGetHandler handles GET /user.
type UserGetHandler struct {
	db *sqlx.DB
}

func (h *UserGetHandler) handle(
	ctx context.Context, username string,
) (*models.Userinfo, error) {
	var (
		log  = zerolog.Ctx(ctx)
		user models.Userinfo
	)

	if err := database.GetContext(ctx, h.db, &user,
		squirrel.Select(models.UserinfoColumns...).
			From(models.UserDBTable).
			Where(squirrel.Eq{models.UserinfoUsernameColumn: username}),
		log,
	); err != nil {
		return nil, err
	}

	if err := HandleUserActivity(ctx, h.db, &user, log); err != nil {
		return nil, err
	}

	return &user, nil
}

// Handle handles the incoming request.
func (h *UserGetHandler) Handle(
	params user_management.UserGetParams, principal *models.Principal,
) middleware.Responder {
	ctx := params.HTTPRequest.Context()

	log := zerolog.Ctx(ctx)
	if !principal.IsAdmin {
		log.Error().Msg("only admins can get specific user informations")

		return user_management.NewUserGetUnauthorized()
	}

	user, err := h.handle(params.HTTPRequest.Context(), params.Username)
	if err != nil {
		return InternalError(err, log, false)
	}

	return user_management.NewUserGetOK().WithPayload(user)
}
